from scanner import Scanner
from cmn_types import *

import concurrent.futures
from time import sleep


def sensor_found(scanner, sensors):
    for index in range(len(sensors)):
        print('Sensor found: %s' % sensors[index])


def on_sensor_state_changed(sensor, state):
    print('Sensor {0} is {1}'.format(sensor.name, state))


def on_battery_changed(sensor, battery):
    print('Battery: {0}'.format(battery))


def on_signal_data_received(sensor, data):
    print(data)


def on_resist_data_received(sensor, data):
    print(data)


def on_electrodes_state_changed(sensor, data):
    print(data)


def on_envelope_received(sensor, data):
    print(data)


def on_respiration_data_received(sensor, data):
    print(data)


try:
    scanner = Scanner([SensorFamily.SensorLEBrainBitBlack, SensorFamily.SensorLEBrainBit,
                       SensorFamily.SensorLECallibri])

    scanner.sensorsChanged = sensor_found
    scanner.start()
    print("Starting search for 5 sec...")
    sleep(5)
    scanner.stop()

    sensorsInfo = scanner.sensors()
    for i in range(len(sensorsInfo)):
        current_sensor_info = sensorsInfo[i]
        print(sensorsInfo[i])


        def device_connection(sensor_info):
            return scanner.create_sensor(sensor_info)


        with concurrent.futures.ThreadPoolExecutor() as executor:
            future = executor.submit(device_connection, current_sensor_info)
            sensor = future.result()
            print("Device connected")

        sensor.sensorStateChanged = on_sensor_state_changed
        sensor.batteryChanged = on_battery_changed

        sensFamily = sensor.sens_family

        print(sensFamily)
        print(sensor.features)
        print(sensor.commands)
        print(sensor.parameters)
        print(sensor.name)
        print(sensor.state)
        print(sensor.address)
        print(sensor.serial_number)
        print(sensor.batt_power)
        print(sensor.sampling_frequency)
        print(sensor.gain)
        print(sensor.data_offset)
        print(sensor.version)

        if sensor.is_supported_feature(SensorFeature.FeatureSignal):
            sensor.signalDataReceived = on_signal_data_received

        if sensor.is_supported_feature(SensorFeature.FeatureResist):
            sensor.resistDataReceived = on_resist_data_received

        if sensor.is_supported_feature(SensorFeature.FeatureRespiration):
            sensor.respirationDataReceived = on_respiration_data_received

        if sensor.is_supported_feature(SensorFeature.FeatureEnvelope):
            sensor.envelopeDataReceived = on_envelope_received

        if sensor.is_supported_command(SensorCommand.CommandStartSignal):
            sensor.exec_command(SensorCommand.CommandStartSignal)
            print("Start signal")
            sleep(5)
            sensor.exec_command(SensorCommand.CommandStopSignal)
            print("Stop signal")

        if sensor.is_supported_command(SensorCommand.CommandStartResist):
            sensor.exec_command(SensorCommand.CommandStartResist)
            print("Start resist")
            sleep(5)
            sensor.exec_command(SensorCommand.CommandStopResist)
            print("Stop resist")

        if sensor.is_supported_command(SensorCommand.CommandStartEnvelope):
            sensor.exec_command(SensorCommand.CommandStartEnvelope)
            print("Start envelope")
            sleep(5)
            sensor.exec_command(SensorCommand.CommandStopEnvelope)
            print("Stop envelope")

        if sensor.is_supported_command(SensorCommand.CommandStartRespiration):
            sensor.exec_command(SensorCommand.CommandStartRespiration)
            print("Start respiration")
            sleep(5)
            sensor.exec_command(SensorCommand.CommandStopRespiration)
            print("Stop respiration")

        if sensor.is_supported_command(SensorCommand.CommandStartCurrentStimulation):
            sensor.exec_command(SensorCommand.CommandStartCurrentStimulation)
            print("Start CurrentStimulation")
            sleep(5)
            sensor.exec_command(SensorCommand.CommandStopCurrentStimulation)
            print("Stop CurrentStimulation")

        sensor.disconnect()
        print("Disconnect from sensor")
        del sensor

    del scanner
    print('Remove scanner')
except Exception as err:
    print(err)



import contextlib

from neurosdk.__utils import raise_exception_if
from neurosdk.__cmn_types import *
from neurosdk.cmn_types import *

import platform
import pathlib
import sys

_libname = None
if sys.platform == "win32":
    arc = platform.architecture()
    if arc[0].__contains__("64"):
        _libname = pathlib.Path(__file__).parent.resolve() / "libs" / "neurosdk2-x64.dll"
    else:
        _libname = pathlib.Path(__file__).parent.resolve() / "libs" / "neurosdk2-x32.dll"
elif sys.platform.startswith("linux"):
    print('Add linux lib')
elif sys.platform == "darwin":
    print('Add macos lib')
else:
    raise Exception("This platform (%s) is currently not supported by py_neurosdk." % sys.platform)

_neuro_lib = CDLL(str(_libname))


class Sensor:
    def __init__(self, ptr):
        """
        Don't use it to creation device

        -----------
        :param ptr:
            Inner pointer to Sensor object
        """
        # signatures
        _neuro_lib.freeSensor.argtypes = [SensorPointer]
        _neuro_lib.freeSensor.restype = c_void_p
        _neuro_lib.connectSensor.argtypes = [SensorPointer, POINTER(OpStatus)]
        _neuro_lib.connectSensor.restype = c_uint8
        _neuro_lib.disconnectSensor.argtypes = [SensorPointer, POINTER(OpStatus)]
        _neuro_lib.disconnectSensor.restype = c_uint8
        _neuro_lib.getFeaturesCountSensor.argtypes = [SensorPointer]
        _neuro_lib.getFeaturesCountSensor.restype = c_int32
        _neuro_lib.getFeaturesSensor.argtypes = [SensorPointer, POINTER(c_int8), POINTER(c_int32), POINTER(OpStatus)]
        _neuro_lib.getFeaturesSensor.restype = c_uint8
        _neuro_lib.isSupportedFeatureSensor.argtypes = [SensorPointer, c_int8]
        _neuro_lib.isSupportedFeatureSensor.restype = c_int8
        _neuro_lib.getCommandsCountSensor.argtypes = [SensorPointer]
        _neuro_lib.getCommandsCountSensor.restype = c_int32
        _neuro_lib.getCommandsSensor.argtypes = [SensorPointer, POINTER(c_int8), POINTER(c_int32), POINTER(OpStatus)]
        _neuro_lib.getCommandsSensor.restype = c_uint8
        _neuro_lib.isSupportedCommandSensor.argtypes = [SensorPointer, c_int8]
        _neuro_lib.isSupportedCommandSensor.restype = c_int8
        _neuro_lib.getParametersCountSensor.argtypes = [SensorPointer]
        _neuro_lib.getParametersCountSensor.restype = c_int32
        _neuro_lib.getParametersSensor.argtypes = [SensorPointer, POINTER(NativeParameterInfo), POINTER(c_int32), 
                                                   POINTER(OpStatus)]
        _neuro_lib.getParametersSensor.restype = c_uint8
        _neuro_lib.isSupportedParameterSensor.argtypes = [SensorPointer, c_int8]
        _neuro_lib.isSupportedParameterSensor.restype = c_int8
        _neuro_lib.getChannelsCountSensor.argtypes = [SensorPointer]
        _neuro_lib.getChannelsCountSensor.restype = c_int32
        _neuro_lib.execCommandSensor.argtypes = [SensorPointer, c_int8, POINTER(OpStatus)]
        _neuro_lib.execCommandSensor.restype = c_uint8
        _neuro_lib.getFamilySensor.argtypes = [SensorPointer]
        _neuro_lib.getFamilySensor.restype = c_int8
        _neuro_lib.readNameSensor.argtypes = [SensorPointer, c_char_p, c_int32, POINTER(OpStatus)]
        _neuro_lib.readNameSensor.restype = c_uint8
        _neuro_lib.writeNameSensor.argtypes = [SensorPointer, c_char_p, c_int32, POINTER(OpStatus)]
        _neuro_lib.writeNameSensor.restype = c_uint8
        _neuro_lib.readStateSensor.argtypes = [SensorPointer, POINTER(c_int8), POINTER(OpStatus)]
        _neuro_lib.readStateSensor.restype = c_uint8
        _neuro_lib.readAddressSensor.argtypes = [SensorPointer, c_char_p, c_int32, POINTER(OpStatus)]
        _neuro_lib.readAddressSensor.restype = c_uint8
        _neuro_lib.readSerialNumberSensor.argtypes = [SensorPointer, c_char_p, c_int32, POINTER(OpStatus)]
        _neuro_lib.readSerialNumberSensor.restype = c_uint8
        _neuro_lib.writeSerialNumberSensor.argtypes = [SensorPointer, c_char_p, c_int32, POINTER(OpStatus)]
        _neuro_lib.writeSerialNumberSensor.restype = c_uint8
        _neuro_lib.readBattPowerSensor.argtypes = [SensorPointer, POINTER(c_int32), POINTER(OpStatus)]
        _neuro_lib.readBattPowerSensor.restype = c_uint8
        _neuro_lib.readSamplingFrequencySensor.argtypes = [SensorPointer, POINTER(c_int8), POINTER(OpStatus)]
        _neuro_lib.readSamplingFrequencySensor.restype = c_uint8
        _neuro_lib.readGainSensor.argtypes = [SensorPointer, POINTER(c_int8), POINTER(OpStatus)]
        _neuro_lib.readGainSensor.restype = c_uint8
        _neuro_lib.readDataOffsetSensor.argtypes = [SensorPointer, POINTER(c_uint8), POINTER(OpStatus)]
        _neuro_lib.readDataOffsetSensor.restype = c_uint8
        _neuro_lib.readFirmwareModeSensor.argtypes = [SensorPointer, POINTER(c_int8), POINTER(OpStatus)]
        _neuro_lib.readFirmwareModeSensor.restype = c_uint8
        _neuro_lib.readVersionSensor.argtypes = [SensorPointer, POINTER(NativeSensorVersion), POINTER(OpStatus)]
        _neuro_lib.readVersionSensor.restype = c_uint8
        _neuro_lib.addBatteryCallback.argtypes = [SensorPointer, BatteryCallback, c_void_p, ctypes.py_object, 
                                                  POINTER(OpStatus)]
        _neuro_lib.addBatteryCallback.restype = c_uint8
        _neuro_lib.removeBatteryCallback.argtypes = [BattPowerListenerHandle]
        _neuro_lib.removeBatteryCallback.restype = c_void_p
        _neuro_lib.addConnectionStateCallback.argtypes = [SensorPointer, ConnectionStateCallback, c_void_p,
                                                          ctypes.py_object, POINTER(OpStatus)]
        _neuro_lib.addConnectionStateCallback.restype = c_uint8
        _neuro_lib.removeConnectionStateCallback.argtypes = [SensorStateListenerHandle]
        _neuro_lib.removeConnectionStateCallback.restype = c_void_p

        self.sensor_ptr = ptr

        self.sensorStateChanged = None
        self.__add_connection_state_callback()

        if self.is_supported_parameter(SensorParameter.ParameterBattPower):
            self.batteryChanged = None
            self.__add_battery_callback()

        self.__closed = False

    def __del__(self):
        with contextlib.suppress(Exception):
            if not self.__closed:
                self.__closed = True

                self.batteryChanged = None
                self.sensorStateChanged = None

                _neuro_lib.removeBatteryCallback(self.__batteryCallbackHandle)
                _neuro_lib.removeConnectionStateCallback(self.__connectionStateCallbackHandle)
                _neuro_lib.freeSensor(self.sensor_ptr)

                self.sensor_ptr = None

    def __add_battery_callback(self):
        def __py_battery_callback(ptr, battery, user_data):
            if user_data.batteryChanged is not None:
                user_data.batteryChanged(user_data, int(battery))

        status = OpStatus()
        self.__batteryCallback = BatteryCallback(__py_battery_callback)
        self.__batteryCallbackHandle = BattPowerListenerHandle()
        _neuro_lib.addBatteryCallback(self.sensor_ptr, self.__batteryCallback, byref(self.__batteryCallbackHandle),
                                      py_object(self), byref(status))
        raise_exception_if(status)

    def __add_connection_state_callback(self):
        def __py_connection_state_callback(ptr, state, user_data):
            if user_data.sensorStateChanged is not None:
                user_data.sensorStateChanged(user_data, SensorState(state))

        status = OpStatus()
        self.__connectionStateCallback = ConnectionStateCallback(__py_connection_state_callback)
        self.__connectionStateCallbackHandle = SensorStateListenerHandle()
        _neuro_lib.addConnectionStateCallback(self.sensor_ptr, self.__connectionStateCallback,
                                              byref(self.__connectionStateCallbackHandle),
                                              py_object(self), byref(status))
        raise_exception_if(status)

    @property
    def sens_family(self) -> SensorFamily:
        status = OpStatus()
        family = _neuro_lib.getFamilySensor(self.sensor_ptr, byref(status))
        return SensorFamily(family)

    @property
    def features(self) -> [SensorFeature]:
        status = OpStatus()
        sz = _neuro_lib.getFeaturesCountSensor(self.sensor_ptr)
        sz_sensor_feature_in_out = SizeType(c_int32(sz))
        features_val = EnumType(c_int8(sz))
        _neuro_lib.getFeaturesSensor(self.sensor_ptr, features_val, sz_sensor_feature_in_out, byref(status))
        raise_exception_if(status)
        return [SensorFeature(features_val[i]) for i in range(sz_sensor_feature_in_out.contents.value)]

    @property
    def commands(self) -> [SensorCommand]:
        status = OpStatus()
        sz = _neuro_lib.getCommandsCountSensor(self.sensor_ptr)
        sz_sensor_commands_in_out = SizeType(c_int32(sz))
        commands_val = EnumType(c_int8())
        _neuro_lib.getCommandsSensor(self.sensor_ptr, commands_val, sz_sensor_commands_in_out,
                                     byref(status))
        raise_exception_if(status)
        return [SensorCommand(commands_val[i]) for i in range(sz_sensor_commands_in_out.contents.value)]

    @property
    def parameters(self) -> [ParameterInfo]:
        status = OpStatus()
        sz = _neuro_lib.getParametersCountSensor(self.sensor_ptr)
        sz_sensor_parameters_in_out = SizeType(c_int32(sz))
        parameters_val = (NativeParameterInfo * sz)()
        _neuro_lib.getParametersSensor(self.sensor_ptr, parameters_val, sz_sensor_parameters_in_out,
                                       byref(status))
        raise_exception_if(status)
        return [ParameterInfo(Param=SensorParameter(parameters_val[i].Param),
                              ParamAccess=SensorParamAccess(parameters_val[i].ParamAccess)) for i in
                range(sz_sensor_parameters_in_out.contents.value)]

    @property
    def name(self) -> str:
        status = OpStatus()
        name_out = ctypes.create_string_buffer(SENSOR_NAME_LEN)
        _neuro_lib.readNameSensor(self.sensor_ptr, name_out, SENSOR_NAME_LEN, byref(status))
        raise_exception_if(status)
        return ''.join([chr(c) for c in name_out.value]).rstrip('\x00')

    @name.setter
    def name(self, value: str):
        status = OpStatus()
        _neuro_lib.writeNameSensor(self.sensor_ptr, value.encode('utf-8'), len(value), byref(status))
        raise_exception_if(status)

    @property
    def state(self) -> SensorState:
        status = OpStatus()
        state_val = EnumType(c_int8(1))
        _neuro_lib.readStateSensor(self.sensor_ptr, state_val, byref(status))
        raise_exception_if(status)
        return SensorState(state_val.contents.value)

    @property
    def address(self) -> str:
        status = OpStatus()
        address_out = ctypes.create_string_buffer(SENSOR_ADR_LEN)
        _neuro_lib.readAddressSensor(self.sensor_ptr, address_out, SENSOR_ADR_LEN, byref(status))
        raise_exception_if(status)
        return ''.join([chr(c) for c in address_out.value]).rstrip('\x00')

    @property
    def serial_number(self) -> str:
        status = OpStatus()
        serial_number_out = ctypes.create_string_buffer(SENSOR_SN_LEN)
        _neuro_lib.readSerialNumberSensor(self.sensor_ptr, serial_number_out, SENSOR_SN_LEN, byref(status))
        raise_exception_if(status)
        return ''.join([chr(c) for c in serial_number_out.value]).rstrip('\x00')

    @serial_number.setter
    def serial_number(self, sn: str):
        status = OpStatus()
        _neuro_lib.writeSerialNumberSensor(self.sensor_ptr, sn.encode('utf-8'), len(sn), byref(status))
        raise_exception_if(status)

    @property
    def batt_power(self) -> int:
        status = OpStatus()
        power = SizeType(c_int32(1))
        _neuro_lib.readBattPowerSensor(self.sensor_ptr, power, byref(status))
        raise_exception_if(status)
        return int(power.contents.value)

    @property
    def sampling_frequency(self) -> SensorSamplingFrequency:
        if self.is_supported_parameter(SensorParameter.ParameterSamplingFrequency):
            status = OpStatus()
            sf_val = EnumType(c_int8(1))
            _neuro_lib.readSamplingFrequencySensor(self.sensor_ptr, sf_val, byref(status))
            raise_exception_if(status)
            return SensorSamplingFrequency(sf_val.contents.value)
        return SensorSamplingFrequency.FrequencyUnsupported

    @property
    def version(self) -> SensorVersion:
        status = OpStatus()
        svp = POINTER(NativeSensorVersion)
        version = svp(NativeSensorVersion())
        _neuro_lib.readVersionSensor(self.sensor_ptr, version, byref(status))
        raise_exception_if(status)
        return SensorVersion(FwMajor=int(version.contents.FwMajor),
                             FwMinor=int(version.contents.FwMinor),
                             FwPatch=int(version.contents.FwPatch),
                             HwMajor=int(version.contents.HwMajor),
                             HwMinor=int(version.contents.HwMinor),
                             HwPatch=int(version.contents.HwPatch),
                             ExtMajor=int(version.contents.ExtMajor))

    @property
    def gain(self) -> SensorGain:
        status = OpStatus()
        gain_val = EnumType(c_int8(1))
        _neuro_lib.readGainSensor(self.sensor_ptr, gain_val, byref(status))
        raise_exception_if(status)
        return SensorGain(gain_val.contents.value)

    @property
    def data_offset(self) -> SensorDataOffset:
        if self.is_supported_parameter(SensorParameter.ParameterOffset):
            status = OpStatus()
            cp = POINTER(c_uint8)
            data_offset_val = cp(c_uint8(1))
            _neuro_lib.readDataOffsetSensor(self.sensor_ptr, data_offset_val, byref(status))
            raise_exception_if(status)
            return SensorDataOffset(data_offset_val.contents.value)
        return SensorDataOffset.DataOffsetUnsupported

    @property
    def firmware_mode(self) -> SensorFirmwareMode:
        status = OpStatus()
        firmware_mode_val = EnumType(c_int8(1))
        _neuro_lib.readFirmwareModeSensor(self.sensor_ptr, firmware_mode_val, byref(status))
        raise_exception_if(status)
        return SensorFirmwareMode(firmware_mode_val.contents.value)

    def connect(self):
        """
        Device connections. After a successful connection, the sensorStateChanged callback will be called. It is a
        blocking method.

        :raises BaseException:
            If an internal error occurred while connecting.
        """
        status = OpStatus()
        _neuro_lib.connectSensor(self.sensor_ptr, byref(status))
        raise_exception_if(status)

    def disconnect(self):
        """
        Disconnect from the device. After a successful shutdown, the sensorStateChanged callback will be called

        :raises BaseException:
            If an internal error occurred while disconnecting.
        """
        status = OpStatus()
        _neuro_lib.disconnectSensor(self.sensor_ptr, byref(status))
        raise_exception_if(status)

    def is_supported_feature(self, future: SensorFeature) -> bool:
        """
        Checks if a feature is supported

        :param future:
            Feature to be checked for support
        :return: bool
            Is the feature supported or not
        """
        status = OpStatus()
        supported = _neuro_lib.isSupportedFeatureSensor(self.sensor_ptr, future.value, byref(status))
        return bool(int(supported))

    def is_supported_command(self, sensor_command: SensorCommand) -> bool:
        """
        Checks if a command is supported

        :param sensor_command:
            Command to be checked for support
        :return:
            Is the command supported or not
        """
        status = OpStatus()
        supported = _neuro_lib.isSupportedCommandSensor(self.sensor_ptr, sensor_command.value, byref(status))
        return bool(int(supported))

    def is_supported_parameter(self, sensor_parameter: SensorParameter) -> bool:
        """
        Checks if a parameter is supported

        :param sensor_parameter:
            Parameter to be checked for support
        :return:
            Is the parameter supported or not
        """
        status = OpStatus()
        supported = _neuro_lib.isSupportedParameterSensor(self.sensor_ptr, sensor_parameter.value, byref(status))
        return bool(int(supported))

    def exec_command(self, sensor_command: SensorCommand):
        """
        Sends a specific command to the device.It is a blocking method.

        :param sensor_command: SensorCommand
            Command to execute

        :raises BaseException:
            If an internal error occurred while executing command.
        """
        status = OpStatus()
        _neuro_lib.execCommandSensor(self.sensor_ptr, sensor_command.value, byref(status))
        raise_exception_if(status)



import contextlib
from abc import abstractmethod, ABC

from neurosdk.__utils import raise_exception_if
from neurosdk.__cmn_types import *
from neurosdk.cmn_types import *
from neurosdk.sensor import Sensor, _neuro_lib


class ResistSensor(Sensor, ABC):
    def __init__(self, ptr):
        super().__init__(ptr)
        # signatures
        if super().is_supported_parameter(SensorParameter.ParameterSamplingFrequencyResist):
            _neuro_lib.readSamplingFrequencyResistSensor.argtypes = [SensorPointer, POINTER(c_int8), POINTER(OpStatus)]
            _neuro_lib.readSamplingFrequencyResistSensor.restype = c_uint8
        if self.is_supported_feature(SensorFeature.FeatureResist):
            self.resistDataReceived = None
            self.set_resist_callbacks()
        self.__closed = False

    def __del__(self):
        with contextlib.suppress(Exception):
            if not self.__closed:
                self.__closed = True
                self.resistDataReceived = None
                self.unset_resist_callbacks()
        super().__del__()

    @property
    def sampling_frequency_resist(self) -> SensorSamplingFrequency:
        if super().is_supported_parameter(SensorParameter.ParameterSamplingFrequencyResist):
            status = OpStatus()
            sampling_frequency_out = EnumType(c_int8(1))
            _neuro_lib.readSamplingFrequencyResistSensor(self.sensor_ptr, sampling_frequency_out)
            raise_exception_if(status)
            return SensorSamplingFrequency(sampling_frequency_out.contents.value)
        return SensorSamplingFrequency.FrequencyUnsupported

    @abstractmethod
    def set_resist_callbacks(self):
        pass

    @abstractmethod
    def unset_resist_callbacks(self):
        pass



import contextlib

from neurosdk.__utils import raise_exception_if
from neurosdk.__cmn_types import *
from neurosdk.cmn_types import *
from neurosdk.sensor import Sensor, _neuro_lib


class MemsSensor(Sensor):
    def __init__(self, ptr):
        super().__init__(ptr)
        # signatures
        _neuro_lib.readAccelerometerSensSensor.argtypes = [SensorPointer, POINTER(c_int8), POINTER(OpStatus)]
        _neuro_lib.readAccelerometerSensSensor.restype = c_uint8
        _neuro_lib.writeAccelerometerSensSensor.argtypes = [SensorPointer, c_int8, POINTER(OpStatus)]
        _neuro_lib.writeAccelerometerSensSensor.restype = c_uint8
        _neuro_lib.readGyroscopeSensSensor.argtypes = [SensorPointer, POINTER(c_int8), POINTER(OpStatus)]
        _neuro_lib.readGyroscopeSensSensor.restype = c_uint8
        _neuro_lib.writeGyroscopeSensSensor.argtypes = [SensorPointer, c_int8, POINTER(OpStatus)]
        _neuro_lib.writeGyroscopeSensSensor.restype = c_uint8
        _neuro_lib.readSamplingFrequencyMEMSSensor.argtypes = [SensorPointer, POINTER(c_int8), POINTER(OpStatus)]
        _neuro_lib.readSamplingFrequencyMEMSSensor.restype = c_uint8
        _neuro_lib.addMEMSDataCallback.argtypes = [SensorPointer, MEMSDataCallback, c_void_p, ctypes.py_object,
                                                   POINTER(OpStatus)]
        _neuro_lib.addMEMSDataCallback.restype = c_uint8
        _neuro_lib.removeMEMSDataCallback.argtypes = [MEMSDataListenerHandle]
        _neuro_lib.removeMEMSDataCallback.restype = c_void_p
        if self.is_supported_feature(SensorFeature.FeatureMEMS):
            self.memsDataReceived = None
            self.__add_mems_data_callback()

        self.__closed = False

    def __del__(self):
        with contextlib.suppress(Exception):
            if not self.__closed:
                self.__closed = True
                self.memsDataReceived = None
                _neuro_lib.removeMEMSDataCallback(self.__memsDataCallbackHandle)
        super().__del__()

    @property
    def sampling_frequency_mems(self) -> SensorSamplingFrequency:
        status = OpStatus()
        sampling_frequency_out = EnumType(c_int8(1))
        _neuro_lib.readSamplingFrequencyMEMSSensor(self.sensor_ptr, sampling_frequency_out, byref(status))
        raise_exception_if(status)
        return SensorSamplingFrequency(sampling_frequency_out.contents.value)

    @property
    def acc_sens(self) -> SensorAccelerometerSensitivity:
        status = OpStatus()
        acc_sens_val = EnumType(c_int8(1))
        _neuro_lib.readAccelerometerSensSensor(self.sensor_ptr, acc_sens_val, byref(status))
        raise_exception_if(status)
        return SensorAccelerometerSensitivity(acc_sens_val.contents.value)

    @property
    def gyro_sens(self) -> SensorGyroscopeSensitivity:
        status = OpStatus()
        gyro_sens_val = EnumType(c_int8(1))
        _neuro_lib.readGyroscopeSensSensor(self.sensor_ptr, gyro_sens_val, byref(status))
        raise_exception_if(status)
        return SensorGyroscopeSensitivity(gyro_sens_val.contents.value)

    def __add_mems_data_callback(self):
        def __py_mems_data_callback(ptr, data, sz_data, user_data):
            mems_data = [MEMSData(PackNum=int(data[i].PackNum),
                                  Accelerometer=Point3D(X=data[i].Accelerometer.X,
                                                        Y=data[i].Accelerometer.Y,
                                                        Z=data[i].Accelerometer.Z),
                                  Gyroscope=Point3D(X=data[i].Gyroscope.X,
                                                    Y=data[i].Gyroscope.Y,
                                                    Z=data[i].Gyroscope.Z)) for i in range(sz_data)]
            if user_data.memsDataReceived is not None:
                user_data.memsDataReceived(user_data, mems_data)

        status = OpStatus()
        self.__memsDataCallback = MEMSDataCallback(__py_mems_data_callback)
        self.__memsDataCallbackHandle = MEMSDataListenerHandle()
        _neuro_lib.addMEMSDataCallback(self.sensor_ptr, self.__memsDataCallback,
                                       byref(self.__memsDataCallbackHandle), py_object(self), byref(status))
        raise_exception_if(status)



import contextlib
from abc import abstractmethod, ABC

from neurosdk.__cmn_types import *
from neurosdk.cmn_types import *
from neurosdk.sensor import Sensor


class SignalSensor(Sensor, ABC):
    def __init__(self, ptr):
        super().__init__(ptr)
        if self.is_supported_feature(SensorFeature.FeatureSignal):
            self.signalDataReceived = None
            self.set_signal_callbacks()
        self.__closed = False

    def __del__(self):
        with contextlib.suppress(Exception):
            if not self.__closed:
                self.__closed = True
                self.signalDataReceived = None
                self.unset_signal_callbacks()
        super().__del__()

    @abstractmethod
    def set_signal_callbacks(self):
        pass

    @abstractmethod
    def unset_signal_callbacks(self):
        pass



import contextlib
from abc import abstractmethod, ABC

from neurosdk.__utils import raise_exception_if
from neurosdk.__cmn_types import *
from neurosdk.cmn_types import *
from neurosdk.sensor import Sensor, _neuro_lib


class EnvelopeSensor(Sensor, ABC):
    def __init__(self, ptr):
        super().__init__(ptr)
        # signatures
        _neuro_lib.readSamplingFrequencyEnvelopeSensor.argtypes = [SensorPointer, POINTER(c_int8), POINTER(OpStatus)]
        _neuro_lib.readSamplingFrequencyEnvelopeSensor.restype = c_uint8
        if self.is_supported_feature(SensorFeature.FeatureEnvelope):
            self.envelopeDataReceived = None
            self.set_envelope_callbacks()
        self.__closed = False

    def __del__(self):
        with contextlib.suppress(Exception):
            if not self.__closed:
                self.__closed = True
                self.envelopeDataReceived = None
                self.unset_envelope_callbacks()
        super().__del__()

    @property
    def sampling_frequency_envelope(self) -> SensorSamplingFrequency:
        status = OpStatus()
        sf_val = EnumType(c_int8(1))
        _neuro_lib.readSamplingFrequencyEnvelopeSensor(self.sensor_ptr, sf_val, byref(status))
        raise_exception_if(status)
        return SensorSamplingFrequency(sf_val.contents.value)

    @abstractmethod
    def set_envelope_callbacks(self):
        pass

    @abstractmethod
    def unset_envelope_callbacks(self):
        pass



from enum import Enum
from dataclasses import dataclass


class OrderedEnum(Enum):
    def __ge__(self, other):
        if self.__class__ is other.__class__:
            return self.value >= other.value
        return NotImplemented

    def __gt__(self, other):
        if self.__class__ is other.__class__:
            return self.value > other.value
        return NotImplemented

    def __le__(self, other):
        if self.__class__ is other.__class__:
            return self.value <= other.value
        return NotImplemented

    def __lt__(self, other):
        if self.__class__ is other.__class__:
            return self.value < other.value
        return NotImplemented

    def __eq__(self, other):
        if type(self).__qualname__ != type(other).__qualname__:
            return NotImplemented
        return self.name == other.name and self.value == other.value

    def __hash__(self):
        return hash((type(self).__qualname__, self.name))


class SensorFamily(OrderedEnum):
    SensorUnknown = 0
    SensorLECallibri = 1
    SensorLEKolibri = 2
    SensorLEBrainBit = 3
    SensorLEBrainBitBlack = 4


@dataclass
class SensorInfo:
    SensFamily: SensorFamily
    SensModel: int
    Name: str
    Address: str
    SerialNumber: str
    PairingRequired: bool


class SensorFeature(OrderedEnum):
    FeatureSignal = 0
    FeatureMEMS = 1
    FeatureCurrentStimulator = 2
    FeatureRespiration = 3
    FeatureResist = 4
    FeatureFPG = 5
    FeatureEnvelope = 6
    FeaturePhotoStimulator = 7
    FeatureAcousticStimulator = 8


class SensorFirmwareMode(OrderedEnum):
    ModeBootloader = 0
    ModeApplication = 1


class SensorCommand(OrderedEnum):
    CommandStartSignal = 0
    CommandStopSignal = 1
    CommandStartResist = 2
    CommandStopResist = 3
    CommandStartMEMS = 4
    CommandStopMEMS = 5
    CommandStartRespiration = 6
    CommandStopRespiration = 7
    CommandStartCurrentStimulation = 8
    CommandStopCurrentStimulation = 9
    CommandEnableMotionAssistant = 10
    CommandDisableMotionAssistant = 11
    CommandFindMe = 12
    CommandStartAngle = 13
    CommandStopAngle = 14
    CommandCalibrateMEMS = 15
    CommandResetQuaternion = 16
    CommandStartEnvelope = 17
    CommandStopEnvelope = 18
    CommandResetMotionCounter = 19
    CommandCalibrateStimulation = 20
    CommandIdle = 21
    CommandPowerDown = 22
    CommandStartFPG = 23
    CommandStopFPG = 24
    CommandStartSignalAndResist = 25
    CommandStopSignalAndResist = 26
    CommandStartPhotoStimulation = 27
    CommandStopPhotoStimulation = 28
    CommandStartAcousticStimulation = 29
    CommandStopAcousticStimulation = 30


class SensorParameter(OrderedEnum):
    ParameterName = 0
    ParameterState = 1
    ParameterAddress = 2
    ParameterSerialNumber = 3
    ParameterHardwareFilterState = 4
    ParameterFirmwareMode = 5
    ParameterSamplingFrequency = 6
    ParameterGain = 7
    ParameterOffset = 8
    ParameterExternalSwitchState = 9
    ParameterADCInputState = 10
    ParameterAccelerometerSens = 11
    ParameterGyroscopeSens = 12
    ParameterStimulatorAndMAState = 13
    ParameterStimulatorParamPack = 14
    ParameterMotionAssistantParamPack = 15
    ParameterFirmwareVersion = 16
    ParameterMEMSCalibrationStatus = 17
    ParameterMotionCounterParamPack = 18
    ParameterMotionCounter = 19
    ParameterBattPower = 20
    ParameterSensorFamily = 21
    ParameterSensorMode = 22
    ParameterIrAmplitude = 23
    ParameterRedAmplitude = 24
    ParameterEnvelopeAvgWndSz = 25
    ParameterEnvelopeDecimation = 26
    ParameterSamplingFrequencyResist = 27
    ParameterSamplingFrequencyMEMSv = 28
    ParameterSamplingFrequencyFPG = 29
    ParameterAmplifier = 30
    ParameterSensorChannels = 31
    ParameterSamplingFrequencyResp = 32


class SensorParamAccess(OrderedEnum):
    ParamAccessRead = 0
    ParamAccessReadWrite = 1
    ParamAccessReadNotify = 2


class SensorState(OrderedEnum):
    StateInRange = 0
    StateOutOfRange = 1


class SensorSamplingFrequency(OrderedEnum):
    FrequencyHz10 = 0
    FrequencyHz20 = 1
    FrequencyHz100 = 2
    FrequencyHz125 = 3
    FrequencyHz250 = 4
    FrequencyHz500 = 5
    FrequencyHz1000 = 6
    FrequencyHz2000 = 7
    FrequencyHz4000 = 8
    FrequencyHz8000 = 9
    FrequencyUnsupported = 10


class SensorGain(OrderedEnum):
    SensorGain1 = 0
    SensorGain2 = 1
    SensorGain3 = 2
    SensorGain4 = 3
    SensorGain6 = 4
    SensorGain8 = 5
    SensorGain12 = 6
    SensorGain24 = 7
    SensorGainUnsupported = 8


class SensorDataOffset(OrderedEnum):
    DataOffset0 = 0x00
    DataOffset1 = 0x01
    DataOffset2 = 0x02
    DataOffset3 = 0x03
    DataOffset4 = 0x04
    DataOffset5 = 0x05
    DataOffset6 = 0x06
    DataOffset7 = 0x07
    DataOffset8 = 0x08
    DataOffsetUnsupported = 0xFF


@dataclass
class ParameterInfo:
    Param: SensorParameter
    ParamAccess: SensorParamAccess


@dataclass
class SensorVersion:
    FwMajor: int
    FwMinor: int
    FwPatch: int
    HwMajor: int
    HwMinor: int
    HwPatch: int
    ExtMajor: int


@dataclass
class Point3D:
    X: float = 0
    Y: float = 0
    Z: float = 0


@dataclass
class MEMSData:
    PackNum: int
    Accelerometer: Point3D
    Gyroscope: Point3D


class SensorAccelerometerSensitivity(OrderedEnum):
    AccSens2g = 0
    AccSens4g = 1
    AccSens8g = 2
    AccSens16g = 3
    AccSensUnsupported = 4


class SensorGyroscopeSensitivity(OrderedEnum):
    GyroSens250Grad = 0
    GyroSens500Grad = 1
    GyroSens1000Grad = 2
    GyroSens2000Grad = 3
    GyroSensUnsupported = 4

    
@dataclass
class CallibriSignalData:
    PackNum: int
    Samples: [float]


@dataclass
class CallibriRespirationData:
    PackNum: int
    Samples: [float]


@dataclass
class CallibriEnvelopeData:
    PackNum: int
    Sample: float

    
class CallibriStimulatorState(OrderedEnum):
    StimStateNoParams = 0,
    StimStateDisabled = 1,
    StimStateEnabled = 2,
    StimStateUnsupported = 0xFF


class CallibriMotionAssistantLimb(OrderedEnum):
    MALimbRightLeg = 0,
    MALimbLeftLeg = 1,
    MALimbRightArm = 2,
    MALimbLeftArm = 3,
    MALimbUnsupported = 0xFF


class SensorFilter(OrderedEnum):
    FilterHPFBwhLvl1CutoffFreq1Hz = 0
    FilterHPFBwhLvl1CutoffFreq5Hz = 1
    FilterBSFBwhLvl2CutoffFreq45_55Hz = 2
    FilterBSFBwhLvl2CutoffFreq55_65Hz = 3
    FilterHPFBwhLvl2CutoffFreq10Hz = 4
    FilterLPFBwhLvl2CutoffFreq400H = 5


class SensorExternalSwitchInput(OrderedEnum):
    ExtSwInMioElectrodesRespUSB = 0
    ExtSwInMioElectrodes = 1
    ExtSwInMioUSB = 2
    ExtSwInRespUSB = 3


class SensorADCInput(OrderedEnum):
    ADCInputElectrodes = 0
    ADCInputShort = 1
    ADCInputTest = 2
    ADCInputResistance = 3


@dataclass
class CallibriStimulatorMAState:
    StimulatorState: CallibriStimulatorState
    MAState: CallibriStimulatorState


@dataclass
class CallibriStimulationParams:
    Current: int
    PulseWidth: int
    Frequency: int
    StimulusDuration: int


@dataclass
class CallibriMotionAssistantParams:
    GyroStart: int
    GyroStop: int
    Limb: CallibriMotionAssistantLimb
    MinPauseMs: int


@dataclass
class CallibriMotionCounterParam:
    InsenseThresholdMG: int
    InsenseThresholdSample: int


class CallibriColorType(OrderedEnum):
    CallibriColorRed = 0
    CallibriColorYellow = 1
    CallibriColorBlue = 2
    CallibriColorWhite = 3
    CallibriColorUnknown = 4


class CallibriSignalType(OrderedEnum):
    CallibriSignalTypeEEG = 0,
    CallibriSignalTypeEMG = 1,
    CallibriSignalTypeECG = 2,
    CallibriSignalTypeEDA = 3,
    CallibriSignalTypeStrainGaugeBreathing = 4,
    CallibriSignalTypeImpedanceBreathing = 5,
    CallibriSignalTypeUnknown = 6


class CallibriElectrodeState(OrderedEnum):
    ElStNormal = 0
    ElStHighResistance = 1
    ElStDetached = 2


@dataclass
class QuaternionData:
    PackNum: int
    W: float = 0
    X: float = 0
    Y: float = 0
    Z: float = 0


@dataclass
class FPGData:
    PackNum: int
    IrAmplitude: float
    RedAmplitude: float


class IrAmplitude(OrderedEnum):
    IrAmp0 = 0,
    IrAmp14 = 1,
    IrAmp28 = 2,
    IrAmp42 = 3,
    IrAmp56 = 4,
    IrAmp70 = 5,
    IrAmp84 = 6,
    IrAmp100 = 7,
    IrAmpUnsupported = 0xFF


class RedAmplitude(OrderedEnum):
    RedAmp0 = 0,
    RedAmp14 = 1,
    RedAmp28 = 2,
    RedAmp42 = 3,
    RedAmp56 = 4,
    RedAmp70 = 5,
    RedAmp84 = 6,
    RedAmp100 = 7,
    RedAmpUnsupported = 0xFF

    
@dataclass
class BrainBitSignalData:
    PackNum: int
    Marker: int
    O1: float
    O2: float
    T3: float
    T4: float


@dataclass
class BrainBitResistData:
    O1: float
    O2: float
    T3: float
    T4: float

    
class SensorAmpMode(OrderedEnum):
    Invalid = 0
    PowerDown = 1
    Idle = 2
    Signal = 3
    Resist = 4
    SignalResist = 5



from neurosdk.__cmn_types import *
from neurosdk.cmn_types import *
from neurosdk.__cmn_types import OpStatus
from neurosdk.__utils import raise_exception_if
from neurosdk.sensor import Sensor, _neuro_lib


class NeuroSmartSensor(Sensor):
    def __init__(self, ptr):
        super().__init__(ptr)
        # signatures
        _neuro_lib.pingNeuroSmart.argtypes = [SensorPointer, c_uint8, POINTER(OpStatus)]
        _neuro_lib.pingNeuroSmart.restype = c_uint8

    def __del__(self):
        super().__del__()

    def ping_neuro_smart(self, marker: int):
        """
        Allows the user to send a flag to the device, which will then be returned to the data packet as the value of
        the token. Not supported by all devices

        :param marker: int
            Byte marker

        :raises BaseException:
            If an internal error occurred while executing command.
        """
        status = OpStatus()
        _neuro_lib.pingNeuroSmart(self.sensor_ptr, c_uint8(marker), byref(status))
        raise_exception_if(status)
